#include <Servo.h>
#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>

//nRF pin setup
RF24 radio(8, 9);
const byte address[6] = "00001";

union packet {
  float vals[6];
  byte bytes[4];
} dataPacket;

//Servo pin setup
Servo servo_1;
Servo servo_2;
Servo servo_3;
Servo servo_4;

//DC motors pin setup
int enA = 6;
int in1 = 23;
int in2 = 25;
int enB = 7;
int in3 = 27;
int in4 = 29;

// x and y values and speed for dc motors
int  xAxis, yAxis;
int motorSpeedA = 0;
int motorSpeedB = 0;

// set ultrasonic sensors pin mode
const int FrontTrig = 43;
const int FrontEcho = 42;

const int RightTrig = 12;
const int RightEcho = 13;

const int LeftTrig = 35;
const int LeftEcho = 34;

int distance1;
int distance2;
int distance3;

void setup() {

  //setup for nRF
  radio.begin();
  radio.openReadingPipe(0, address);
  radio.setPALevel(RF24_PA_MIN);
  radio.startListening();

  //attaches the servos
  servo_1.attach(2);
  servo_2.attach(3);
  servo_3.attach(4);
  servo_4.attach(5);

  //setup for hBridge
  pinMode(enA, OUTPUT);
  pinMode(enB, OUTPUT);
  pinMode(in1, OUTPUT);
  pinMode(in2, OUTPUT);
  pinMode(in3, OUTPUT);
  pinMode(in4, OUTPUT);


  pinMode(FrontTrig, OUTPUT); // Sets the trigPin as an Output
  pinMode(FrontEcho, INPUT); // Sets the echoPin as an Input

  pinMode(RightTrig, OUTPUT); // Sets the trigPin as an Output
  pinMode(RightEcho, INPUT); // Sets the echoPin as an Input

  pinMode(LeftTrig, OUTPUT); // Sets the trigPin as an Output
  pinMode(LeftEcho, INPUT); // Sets the echoPin as an Input
  Serial.begin(9600);

}

void loop() {

  // checking for distances
  distance1 = checkDistSensors(FrontTrig, FrontEcho);
  distance2 = checkDistSensors(RightTrig, RightEcho);
  distance3 = checkDistSensors(LeftTrig, LeftEcho);

  Serial.print(distance1);
  Serial.print("--");
  Serial.print(distance2);
  Serial.print("--");
  Serial.println(distance3);

  if (radio.available())
  {
    radio.read(&dataPacket.vals, sizeof(dataPacket.vals));
    xAxis = 500;
    yAxis = 500;

    if (distance1 > 12) {
      yAxis = dataPacket.vals[1];
      if (distance2 > 5 && distance3 > 5 ) {

        servo_1.write(dataPacket.vals[2]);
        servo_2.write(dataPacket.vals[3]);
        servo_3.write(dataPacket.vals[4]);
        servo_4.write(dataPacket.vals[5]);

        xAxis = dataPacket.vals[0];
      }
    }
    else if (distance2 > 5 && distance3 > 5 ) {
      xAxis = dataPacket.vals[0];
    }
    runMotors();
  }
}


int checkDistSensors(int trig, int echo) {

  digitalWrite(trig, LOW);
  delayMicroseconds(2);
  digitalWrite(trig, HIGH);
  delayMicroseconds(10);
  digitalWrite(trig, LOW);
  long duration = pulseIn(echo, HIGH);
  int distance = duration * 0.034 / 2;

  return distance;
}


void  runMotors() {

  //___MOTORS___
  //___Y-AXIS: FORWARD AND BACKWARD___
  if (yAxis < 470) {

    //-Y1. BACKWARD
    //--Motor A backward
    digitalWrite(in1, HIGH);
    digitalWrite(in2, LOW);

    //--Motor B backward
    digitalWrite(in3, HIGH);
    digitalWrite(in4, LOW);

    //--Convert
    motorSpeedA = map(yAxis, 470, 0, 0, 255);
    motorSpeedB = map(yAxis, 470, 0, 0, 255);
  }

  //-Y2. FORWARD
  else if (yAxis > 550) {
    //--Motor A forward
    digitalWrite(in1, LOW);
    digitalWrite(in2, HIGH);
    //--Motor B forward
    digitalWrite(in3, LOW);
    digitalWrite(in4, HIGH);

    //--Convert
    motorSpeedA = map(yAxis, 550, 1023, 0, 255);
    motorSpeedB = map(yAxis, 550, 1023, 0, 255);
  }

  //-Y3. MIDDLE
  //--Joystick in middle
  else {
    motorSpeedA = 0;
    motorSpeedB = 0;
  }

  //___X AXIS: LEFT AND RIGHT___

  if (xAxis < 470) {

    //-X1. LEFT

    //--Convert
    int xMapped = map(xAxis, 470, 0, 0, 255);

    //--Move to left - decrease left motor speed, increase right motor speed
    motorSpeedA = motorSpeedA - xMapped;
    motorSpeedB = motorSpeedB + xMapped;

    //--Limit the range from 0 to 255
    if (motorSpeedA < 0) {
      motorSpeedA = 0;
    }
    if (motorSpeedB > 255) {
      motorSpeedB = 255;
    }
  }

  //-X2. RIGHT

  if (xAxis > 550) {

    //--Convert
    int xMapped = map(xAxis, 550, 1023, 0, 255);

    //--Move right - decrease right motor speed, increase left motor speed
    motorSpeedA = motorSpeedA + xMapped;
    motorSpeedB = motorSpeedB - xMapped;

    //--Limit the range from 0 to 255
    if (motorSpeedA > 255) {
      motorSpeedA = 255;
    }
    if (motorSpeedB < 0) {
      motorSpeedB = 0;
    }
  }

  //___PREVENT BUZZING (at low speeds)___
  if (motorSpeedA < 70) {
    motorSpeedA = 0;
  }
  if (motorSpeedB < 70) {
    motorSpeedB = 0;
  }

  //___PWM SIGNALS TO MOTORS___
  analogWrite(enA, motorSpeedA);
  analogWrite(enB, motorSpeedB);
}
